#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <getopt.h>  /* getopt_long() */

#include <fcntl.h>  /* low-level i/o */
#include <unistd.h>
#include <errno.h>
#include <malloc.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include <errno.h>

#include <limits.h>

#include <asm/types.h>  /* for videodev2.h */

#include <linux/videodev2.h>

#include "png_io.h"
#include "tools.h"

#define CLEAR(x) memset(&(x), 0, sizeof(x))

#define IMG 0
#define MIN_HOLD 1
#define PEAK_HOLD 2
#define AVE 3
#define DIF 4
#define HIST 5
#define STRETCH 6

#define FNAME_SIZE 160

struct v4l2_format fmt;

struct buffer {
  void *start;
  size_t length;
};

struct frame {
  unsigned char *img_buffer;
  int width;
  int height;
  time_t timestamp;
  long usec;
  long sum;
};

struct img_stack {
  unsigned long *img_buffer;
  int width;
  int height;
  time_t timestamp;
  long usec;
  int num_imgs;
  unsigned long min_val;
  unsigned long max_val;
};

/* Image frames */

/* Ring memory for triggered capture */
struct frame *frames = NULL;
/* Minimum stack */
struct frame min_hold;
/* Maximum stack */
struct frame peak_hold;
/* Average stack */
struct img_stack ave_stack;

static int img_in_mmap_buffer = 2;
static int img_in_buffer = 5;
static int n = 0;

/* Length of peak-hold/average/min stack in seconds */
static int stack_length = 60;

static int previous_frame = -1;

static char *dev_name = NULL;
static int fd = -1;

// Skip the first frame of the stack, it tends to be empty
unsigned char first_frame = 1;

/* MMAP buffers */
struct buffer *buffers = NULL;
static unsigned int n_buffers = 0;

static int threshold = 100000;

/* Which type of images will be saved */
static int use_min = 0;
static int use_max = 0;
static int use_ave = 0;
static int use_dif = 0;
static int use_trig = 0;

/* Time when the imaging started */
static time_t start_time;

/* Length of this imaging session */
static unsigned long capture_length = ULONG_MAX;

/* Starting time of the stacks */
static time_t stack_timestamp;

/* Output directory path. '/' will be appended automatically */
static char *out_dir = NULL;

/* Prefix, which will be prepended to each filename. '_' will be added. */
static char *img_prefix = NULL;

/* Commandline options */
static const char short_options [] = "c:hn:s:mMadtp:o:l:";

/* Long commandline options */
static const struct option long_options [] = {
  { "capture-device", required_argument, NULL, 'c' }, 
  { "help", no_argument, NULL, 'h' }, 
  { "buffer", required_argument, NULL, 'n' }, 
  { "stack-length", required_argument, NULL, 's' }, 
  { "min", no_argument, NULL, 'm' }, 
  { "max", no_argument, NULL, 'M' }, 
  { "ave", no_argument, NULL, 'a' }, 
  { "dif", no_argument, NULL, 'd' }, 
  { "trig", no_argument, NULL, 't' },
  { "prefix", required_argument, NULL, 'p' }, 
  { "out-dir", required_argument, NULL, 'o' }, 
  { "capture-length", required_argument, NULL, 'l' }, 
  { 0, 0, 0, 0 }
};

